//+------------------------------------------------------------------+
//|                                                   lesson2_EA.mq4 |
//|                          Copyright 2021, MQL4club Software Corp. |
//|                                            https://t.me/MQL4club |
//+------------------------------------------------------------------+
#property copyright "Copyright 2021, MQL4club Software Corp."
#property link      "https://t.me/MQL4club"
#property version   "1.00"
#property strict
#property description " this is comment"
#include <stdlib.mqh>

enum LotSize_Method{
                    Manual,
                    Equity, // (% of Equity)
                    Balance, // (% of Balance)
                    FreeMargin // (% of FreeMargin)
};

//--- fast ma settings
input string Fast_MA_settings = "---------------- Fast_MA_settings  ------------------";
input int Fast_MA_Period = 7;
input ENUM_MA_METHOD Fast_MA_Method = MODE_SMA;
input ENUM_APPLIED_PRICE Fast_MA_Applied = PRICE_CLOSE;
//--- slow ma settings
input string Slow_MA_settings = "---------------- Slow_MA_settings  ------------------";
input int Slow_MA_Period = 21;
input ENUM_MA_METHOD Slow_MA_Method = MODE_SMA;
input ENUM_APPLIED_PRICE Slow_MA_Applied = PRICE_CLOSE;

//--- input parameters
input string Trade_settings = "---------------- Trade_settings  ------------------";
input int      StopLoss=20; 
input int      TakeProfit=50;
input int      Slipage = 30;  // slipage (in points)
input ENUM_TIMEFRAMES timeframe = PERIOD_H1;

input string Lotsize_settings = "---------------- Lotsize_settings  ------------------";
input LotSize_Method lotsize_method = Manual;
input double Manual_Lotsize = 0.01;
input double Risk_Percent = 1.0;

input string Time_settings = "---------------- Time_settings  ------------------";
input bool Time_Control = False;
input int Start_Hour = 3;
input int Start_minute = 0;
input int End_Hour = 12;
input int End_minute = 30;


input int      MagicNumber = 1111;
input string  password = "";  // Please Enter PassWord

double pips = _Point;
int PointToPip = 1;
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
//---

   if(_Digits == 5 || _Digits == 3)
   {
      pips = _Point * 10;
      PointToPip = 10;
   }
   
   #include <Checks.mqh>
   
   Comment("The EA Successfully Loaded");
   
//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//---
   
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
//---
   if(TimeControl() == true)
      CheckForSignal();

  }
//+------------------------------------------------------------------+
//|  CheckForSignal                                                  |
//+------------------------------------------------------------------+
void CheckForSignal()
{
   static datetime candleTime = 0;
   if(candleTime != iTime(_Symbol,timeframe,0))
   {
      double FastMa_1 = iMA(_Symbol,timeframe,Fast_MA_Period,0,Fast_MA_Method,Fast_MA_Applied,1);
      double FastMa_2 = iMA(_Symbol,timeframe,Fast_MA_Period,0,Fast_MA_Method,Fast_MA_Applied,2);
      
      double SlowMa_1 = iMA(_Symbol,timeframe,Slow_MA_Period,0,Slow_MA_Method,Slow_MA_Applied,1);
      double SlowMa_2 = iMA(_Symbol,timeframe,Slow_MA_Period,0,Slow_MA_Method,Slow_MA_Applied,2);
      
      if(FastMa_1 > SlowMa_1 && FastMa_2 < SlowMa_2)
         EnterTrade(OP_BUY);
      else if(FastMa_1 < SlowMa_1 && FastMa_2 > SlowMa_2)
         EnterTrade(OP_SELL);
         
      //--- candle time reset
      candleTime = iTime(_Symbol,timeframe,0);
   }
     
}
//+------------------------------------------------------------------+
//|  EnterTrade                                                      |
//+------------------------------------------------------------------+
void EnterTrade(int order_type)
{
   int err;
   double sl=0 , tp=0 , price = 0;
   color order_color = 0;
   
   if(order_type == OP_BUY)
   {
      price = Ask;
      order_color = clrBlue;
   }
   else if(order_type == OP_SELL)
   {
      price = Bid;
      order_color = clrRed;
   }
   
   double Lot = Manual_Lotsize;
   if(lotsize_method != Manual)
   {
      Lot = LotSizeCalc();
   }
   
   Lot = NormalizeDouble(Lot , 2);
   
   int ticket = OrderSend(_Symbol,order_type,Lot,price,Slipage,0,0,"Ma_Expert",MagicNumber,0,order_color);
   
   if(ticket > -1)
   {
      if(OrderSelect(ticket,SELECT_BY_TICKET))
      {
         if(order_type == OP_BUY)
         {
            sl = OrderOpenPrice() - StopLoss * pips;
            tp = OrderOpenPrice() + TakeProfit * pips;
         }
         else if(order_type == OP_SELL)
         {
            sl = OrderOpenPrice() + StopLoss * pips;
            tp = OrderOpenPrice() - TakeProfit * pips;
         
         }
   
         if(!OrderModify(ticket,OrderOpenPrice(),sl,tp,0,clrNONE))
         {
            err = GetLastError();
            Print("could not Modify position due to : ",err,"  ",ErrorDescription(err));

         }
               
      }
      else
      {
          err = GetLastError();
          Print("could not select position due to : ",err,"  ",ErrorDescription(err));
      }
   }
   else
   {
      err = GetLastError();
      Print("could not open position due to : ",err,"  ",ErrorDescription(err));
   }
   
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double LotSizeCalc()
{
   if(StopLoss <= 0)
   {
      Print("Invalid Stop Loss");
      return(-1);
   }
   
   double lotsize = 0;
   double Tick_Value = MarketInfo(_Symbol,MODE_TICKVALUE) * PointToPip;
   double Account_Size = 0;
   
   if(lotsize_method == FreeMargin)
      Account_Size = AccountFreeMargin();
   else if(lotsize_method == Equity)
      Account_Size = AccountEquity();
   else if(lotsize_method == Balance)
      Account_Size = AccountBalance();
      
   double Risk = Account_Size * Risk_Percent * 0.01;  // 1000 * 1 * 0.01 = 10 $;
   
   if(Tick_Value <= 0)
   {
      Print("Invalid Tick_Value");
      return(-1);
   }
   
   lotsize = (Risk / (StopLoss*1.0) ) / Tick_Value ; 
   lotsize = NormalizeDouble(lotsize , 2);
   
   double minlot = MarketInfo(_Symbol,MODE_MINLOT);
   double maxlot = MarketInfo(_Symbol,MODE_MAXLOT);
   
   if(lotsize <= 0)
      lotsize = minlot;
   else if(lotsize > maxlot)
      lotsize = maxlot;
      
   return lotsize;
}
//+------------------------------------------------------------------+
//| TimeControl
//+------------------------------------------------------------------+
bool TimeControl()
{
   if(Time_Control == false)
      return true;
      
   MqlDateTime TimeCurrentStructure;
   datetime Time_Current = TimeCurrent();
   
   TimeToStruct(Time_Current,TimeCurrentStructure);
   
   long Time_Current_Sec = TimeCurrentStructure.hour * 60 * 60 + TimeCurrentStructure.min * 60;
   
   if((Start_Hour *60*60 + Start_minute *60) > (End_Hour *60*60 + End_minute *60))
   {
      if(Time_Current_Sec >= (Start_Hour *60*60 + Start_minute *60) || Time_Current_Sec < (End_Hour *60*60 + End_minute *60))
         return true;
   }
   else if((Start_Hour *60*60 + Start_minute *60) < (End_Hour *60*60 + End_minute *60))
   {
      if(Time_Current_Sec >= (Start_Hour *60*60 + Start_minute *60) && Time_Current_Sec < (End_Hour *60*60 + End_minute *60))
         return true;

   }
   return false;
}